#include <math.h>	// for NAN
#include <stdio.h>

/* Write a function that computes the minimum of an array of any type.
   Array and comparison function are given as for qsort.
   Return a pointer to the minimal element */

// We derive this step by step

// Version 1: minimum of an array of reals, returns the actual number
double min1(double *a, unsigned cnt)
{
  if (!cnt) return NAN;		// Not-a-number: empty array has no min
  double m = *a;		// *a is a[0]
  for (double v; --cnt; )	// loop down to 1
    if ((v = a[cnt]) < m) m = v;// update min
  return m;
}

// Version 2: minimum of an array of reals, return pointer to min
double *min2(double *a, unsigned cnt)
{
  if (!cnt) return NULL;	// NULL means invalid
  double *m = a;		// address of element 0
  for (double *v = a + cnt; --v > a; )	// set just past end; predecrement
    if (*v < *m) m = v;
  return m;
}

// Version 3: use custom comparison function
double *min3(double *a, unsigned cnt, int (*cmp)(const void *, const void *))
{
  if (!cnt) return NULL;	
  double *m = a;		
  for (double *v = a + cnt; --v > a; )	
    if (cmp(v, m) < 0) m = v;	// cmp takes pointers to elements
  return m;
}

// Version 4: void * (arbitrary type); MUST cast to (char *) for arithmetic
void *min(void *a, unsigned cnt, unsigned size, // 3rd argument is element size
	    	int (*cmp)(const void *, const void *))
{
  if (!cnt) return NULL;
  void *m = a;
  /* a + cnt means cnt ELEMENTS past a; CAN'T DO if elt. size unknown (void *)
     for char *, 1 element = 1 byte => number added means number of bytes */
  for (char *v = (char *)a + cnt * size; (v -= size) > (char *)a; )
    if (cmp(v, m) < 0) m = v;
  return m;
}

typedef int (*cmpfn_t)(const void *, const void *); // type of cmp fn pointer

int cmp_dbl(double *p1, double *p2) {
  double diff = *p1 - *p2;
  return diff < 0 ? -1 : diff ? 1 : 0;
}

#define CNT	5

int main(void)
{
  double t[CNT] = { 1, -1.6, 3.14, -2.5, 2.71 };
  printf("%f\n", min1(t, CNT));
  printf("%f\n", *min2(t, CNT));
  printf("%f\n", *min3(t, CNT, (cmpfn_t)cmp_dbl));
  printf("%f\n", *(double *)min(t, CNT, sizeof(double), (cmpfn_t)cmp_dbl));
}
