#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>

/* A string of parantheses is balanced if it is empty or has the form 
   (s1)s2 where s1 and s2 are strings of balanced parantheses. 
   Each balanced string is given a number: N(empty) = 0,
   N((s1)s2) = 2^N(s1)*(2*N(s2)+1)      // math notation ^ for exponent (not C!)
   a) Write a function that prints the string of balanced parantheses 
   for an unsigned number given as parameter. 
   b) Write a function that reads a string of balanced parantheses from input
   (until the first character that does not match) and returns the number
   corresponding to the string. */

void num2par(unsigned n)	// prints string with given number
{
  if (n) {	// else empty string, print/do nothing
    unsigned p2 = 0;	// exponent of 2 in n
    while (n % 2 == 0) { n /= 2; ++p2; }
    putchar('('); num2par(p2); putchar(')');  // print second part
    num2par(n / 2);	// remaining n is odd, same as (n-1)/2
  }
}

unsigned par2num(void)		// reads a string, returns number
{
  int c = getchar();	// if first char not (, empty is only option
  if (c != '(') {  ungetc(c, stdin); return 0; } // c is not part of empty string read
  // string is (s1)s2 : read s1 & get number, check ), read s2 & get number
  unsigned n1 = 1 << par2num();		// 1 << n is 2 to power n (integer)
  if (getchar() != ')') { puts("missing )"); exit(1); }	// exit program
  return n1 * (2 * par2num() + 1);	// combine according to formula
}

int par2num2(void)	// version 2: return -1 on error
{
  int c = getchar();
  if (c != '(') { ungetc(c, stdin); return 0; }
  int n1 = par2num2();
  if (n1 == -1) return -1;	// bad: don't try rest
  if (getchar() != ')') { puts("missing )"); return -1; }
  int n2 = par2num2();
  return n2 == -1 ? -1 : (1 << n1) * (2*n2 + 1);
}

int okafter(void)	// optional: check that nothing or space follows
{
  int c = getchar();
  return c == EOF || isspace(c);
}

int main(void)
{
  num2par(12); putchar('\n');
  unsigned n = par2num();
  if (okafter()) printf("%u\n", n); else puts("bad extra char");
  return 0;
}
