#include <errno.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>

// print width, height and size of a bitmap file

// reads a 32-bit integer, byte order in memory will be same as in file
// if stored in file as little-endian, will work well on little-endian processor
uint32_t rd32(FILE *f) {	
  uint32_t n;
  if (fread (&n, 4, 1, f) != 1) { perror("error on read"); return errno; }
  return n;
}

int main(int argc, char *argv[])
{
  if (argc != 2) {
    fprintf(stderr, "correct usage: program bitmapfile\n");
    return 1;	// or some other error code
  }
  FILE *fp = fopen(argv[1], "rb");
  if (!fp) { perror("error on open"); return errno; }

  if (fgetc(fp) != 'B' || fgetc(fp) != 'M') {	// check file starts with BM
    fputs("not a bitmap\n", stderr);
    exit(1);
  }
  uint32_t fsize = rd32(fp);	// right after BM: 4-byte file size

  fseek(fp, 0x12, SEEK_SET);	// at offset 0x12, width then height
  uint32_t w = rd32(fp), h = rd32(fp);
  printf("width %u x height %u\n", w, h);

  fseek(fp, 0, SEEK_END);	// seek to end
  long where = ftell(fp);	// at end: value of file cursor is file size
  printf("size in file: %u actual size: %ld\n", fsize, where);

  if (fclose(fp)) { perror("error on close"); return errno; }
  return 0;
}
