#include <stdio.h>
#include <stdlib.h>

/* Given an array a of size n, the *children* of index i are those of the
   indices 2i+1 and 2i+2 which are < n.
   Index i is *proper* if the array value a[i] is no less than the value
   at any existing child indices, and these in turn are proper.
   Write a function that takes an integer array and its size and determines
   (returns 1 for true, 0 for false) if index 0 is proper. */

// This is called the heap property and used in heapsort

int proper(int a[], unsigned n, unsigned i) 
{
  int c1 = 2*i+1, c2 = 2*i+2;	// the two children
  return c1 >= n	// if no child1, also no child2, all ok
    || (a[i] >= a[c1]	// ordering to first child
	// if second child exists, check conditions for it
	&& (c2 >= n || (a[i] >= a[c2] && proper(a, n, c2)))
	&& proper(a, n, c1));	// do recursive checks last
}

int proper_r(int a[], unsigned n) { return proper(a, n, 0); }

// or, can check iteratively over array length

int proper_it(int a[], unsigned n)
{
  if (n)
    for (int i = n; --i ; )	// all the way down to 1
      if (a[i] > a[(i-1)/2]) return 0;
  return 1;
}

int main(void)
{
  int a[] = { 7, 2, 5, 1, 2, 4 };	// change last to 6 -> false
  printf("%d\n", proper_r(a, sizeof(a)/sizeof(int)));
  return 0;
}
