#include <ctype.h>
#include <stdio.h>

/* A balanced parantheses stack is a string of n ( followed by n ), e.g. ((()))
   Recursively: an empty string or (s) where s is a balanced parantheses stack.
   Write a function to check if a string read from input has the right format.
   Next, write a function that returns the value of n.
   You can easily do it with a loop and counter, but it is a good exercise
   to follow the recursive definition, which you need in more complex problems.
*/

int okafter(void) {	// must have nothing or whitespace after the string
  int c = getchar();
  return isspace(c) || c == EOF;
}

int check_it(void) {	// 1 if ok, 0 if not ok (not all closed)
  int c;
  unsigned n = 0;
  while ((c = getchar()) == '(') ++n;
  // here we know last c read is not (
  while (n > 0) {
    if (c != ')') { puts("missing )"); return 0; }
    --n;
    c = getchar();
  }
  if (c != EOF) ungetc(c, stdin);	// put back last c read in loop 1 or 2
  return 1;	// balanced so far, will check after
}

int check_r(void) {	// 1 if ok, 0 if not ok (not all closed)
  int c = getchar();
  if (c != '(') { ungetc(c, stdin); return 1; }	// base case empty
  return check_r() && getchar() == ')'; // ok if good string and ) follow
}

int countpairs(void) {	// counts pairs of (), -1 if unmatched (
  int c = getchar();
  if (c != '(') { ungetc(c, stdin); return 0; }	// empty, return 0
  int n = countpairs();	// counts inner string
  return (n == -1 || getchar() != ')') ? -1 : n + 1;
}

int check(void) { return check_r() && okafter(); }	// or check_it() && ...

int main(void)
{
  // option 1, just check
  // puts (check() ? "ok" : "bad");
  // option 2: count
  int n = countpairs();
  if (n != -1 && okafter()) printf("%d\n", n);
  else puts("bad");
  return 0;
}
