#include <stdio.h>
#include <stdlib.h>
#include <string.h>

typedef struct {
  unsigned deg;
  float coef[];
} vpol_t;	// flexible array member, must malloc whole struct 

typedef struct {
  unsigned deg;
  float *coef;
} ppol_t;	// pointer to coef array, must allocate memory for it

// return pointer to memory allocated for entire struct
vpol_t *create_vpol(unsigned deg, float c[])
{
  vpol_t *p = malloc(sizeof(vpol_t) + sizeof(float[deg+1]));
  if (p) {
    p->deg = deg;
    memcpy(p->coef, c, sizeof(float[deg+1]));
  }
  return p;
}

// can return a structure, has fixed size, must malloc coef array
ppol_t create_ppol(unsigned deg, float c[])
{
  ppol_t p = { deg, malloc(sizeof(vpol_t) + sizeof(float[deg+1])) };
  if (p.coef) memcpy(p.coef, c, sizeof(float[deg+1]));
  return p;
}

// can't pass structure, no fixed size, must pass pointer 
void print_vpol(vpol_t *p)
{
  printf("%.2f", p->coef[0]);
  for (int i = 1; i <= p->deg; ++i)
    printf("%+.2f*X^%d", p->coef[i], i);
  putchar('\n');
}

// can pass structure, since it has fixed size
void print_ppol(ppol_t p)
{
  printf("%.2f", p.coef[0]);
  for (int i = 1; i <= p.deg; ++i)
    printf("%+.2f*X^%d", p.coef[i], i);
  putchar('\n');
}

#define DEG	3

int main(void)
{
  float coef[DEG+1] = { 3.5, 1, -2, 3 };
  ppol_t p = create_ppol(DEG, coef);
  vpol_t *v = create_vpol(DEG, coef);
  print_ppol(p);
  print_vpol(v);
  free(p.coef);		// free coefficient array
  free(v);		// free entire polynomial
}
