#include <errno.h>
#include <stdio.h>

// reads one integer in binary from a file; use with pair program wr1int.c

// reads integer from file f, most significant byte first
// check ferror() after calling
int rd_bend(FILE *f)
{
  int n = 0;
  for (int i = sizeof(int); i--; )
    n = (n << 8) | fgetc(f);		// make room, then add (OR) byte
  return n;
}

// reads integer from file f, least significant byte first
// check ferror() after calling
int rd_lend(FILE *f)
{
  int n = 0;
  for (int i = 0; i < 8 * sizeof(int); i += 8)	// keep shifting one byte right
    n |= fgetc(f) << i;
  return n;
}

int main(void)
{
  FILE *f = fopen("1int", "r");	// open file in binary
  if (!f) {
    perror("can't open file for writing");
    return errno;	// return error code from fopen call
  }

  int n;
  // reads 1 object of sizeof(int) at address of n
  if (fread(&n, sizeof(int), 1, f) != 1) {	// did not read 1 object
    perror("error on first read");
    return errno;
  } else printf("integer read: 0x%x\n", n);

  rewind(f);		// back, read the same bytes
  int nb = rd_bend(f);	// read, LSB first
  rewind(f);
  int nl = rd_lend(f);	// read, MSB first

  if (ferror(f)) {
    perror("error on re-read");
    return errno;
  }

  if (nb == nl) puts("same value from both ends");
  else if (n == nb) puts("system is big-endian");
  else if (n == nl) puts("system is little-endian");
  else puts("should be one or the other!!!");
  
  if (fclose(f)) {	// must check for errors on closing
    perror("error closing file");
    return errno;
  }
  return 0;
}
