#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>

/* A stacked expression is an unsigned number or an expression of the form
   (number op stackedexpr) where op may be + or *. 
   Any whitespace is optional. Example: (3 * ( 12 + (3 * 5)))
   Write a function that reads a stacked expression and returns its value,
   or -1 on error */

int error(char *msg) {
  fprintf(stderr, "%s\n", msg);
  return -1;	// can use directly as: return error("message");
}

int getnosp(void) {	// return first non-whitespace char
  int c;
  while (isspace(c = getchar()));
  return c;
}

unsigned readnat_rc(unsigned r, int c) {
  if (isdigit(c)) return readnat_rc(10*r+c-'0', getchar());
  if (c != EOF) ungetc(c, stdin);	// no sense to put back EOF
  return r;
}

int readnat(void) {	// -1 on error
  int c;
  return isdigit(c = getnosp()) ? readnat_rc(0, c) : -1;
}

int expr(void)		// -1 on error
{
  int c = getnosp();
  if (c == '(') {	// (number op expr)
    int n = readnat();
    if (n == -1) return error("number expected");
    int op = getnosp();
    if (op != '+' && op != '*') return error("bad operator");
    int e = expr();
    if (e == -1) return -1;	// already had error message
    if (getnosp() != ')') return error("missing )");
    return op == '+' ? n + e : n * e;
  } else if (isdigit(c)) return readnat_rc(0, c);
  else return error("number or ( expected");
}

int main(void)
{
  int n = expr();
  if (n != -1) printf("%d\n", n);
  return 0;
}