#include <errno.h>
#include <stdio.h>

// writes integer to file f, most significant byte first (big endian)
// returns 0 on success, nonzero on error
int wr_bend(int n, FILE *f)
{
  for (int i = 8 * sizeof(int); (i -= 8) >= 0;)	// shift each byte to LSB
    fputc((n >> i) & 0xFF, f);
  return ferror(f);
}

// writes integer to file f, least significant byte first (little endian)
// returns 0 on success, nonzero on error
int wr_lend(int n, FILE *f)
{
  for (int i = sizeof(int); i--; n >>= 8)	// keep shifting one byte right
    fputc(n & 0xFF, f);
  return ferror(f);
}

int main(void)
{
  FILE *f = fopen("1int", "wb");	// open file in binary
  if (!f) {
    perror("can't open file for writing");
    return errno;	// return error code from fopen call
  }

  int n = 0x134;	// 308 decimal
  // write 1 object of sizeof(int) from address of n
  // writes the number in binary, in the same byte order as in memory
  // for Intel architectures: little endian, i.e. least significant byte first
  if (fwrite(&n, sizeof(int), 1, f) != 1) {	// did not write 1 object
    perror("write error");
    return errno;
  }

  // or: write with wr_bend(n, f), wr_lend(n, f)
  
  if (fclose(f)) {	// must check for errors on closing
    perror("error closing file");
    return errno;
  }
  return 0;
}
